"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemplateUrlsResolver = exports.getTemplateUrls = exports.getStyleUrls = exports.StyleUrlsResolver = void 0;
const path_1 = require("path");
const ts_morph_1 = require("ts-morph");
const vite_1 = require("vite");
class StyleUrlsResolver {
    // These resolvers may be called multiple times during the same
    // compilation for the same files. Caching is required because these
    // resolvers use synchronous system calls to the filesystem, which can
    // degrade performance when running compilations for multiple files.
    styleUrlsCache = new Map();
    resolve(code, id) {
        // Given the code is the following:
        // @Component({
        //   styleUrls: [
        //     './app.component.scss'
        //   ]
        // })
        // The `matchedStyleUrls` would result in: `styleUrls: [\n    './app.component.scss'\n  ]`.
        const matchedStyleUrls = getStyleUrls(code);
        const entry = this.styleUrlsCache.get(id);
        // We're using `matchedStyleUrls` as a key because the code may be changing continuously,
        // resulting in the resolver being called multiple times. While the code changes, the
        // `styleUrls` may remain constant, which means we should always return the previously
        // resolved style URLs.
        if (entry && entry.matchedStyleUrls === matchedStyleUrls) {
            return entry.styleUrls;
        }
        const styleUrls = matchedStyleUrls.map((styleUrlPath) => {
            return `${styleUrlPath}|${(0, vite_1.normalizePath)((0, path_1.resolve)((0, path_1.dirname)(id), styleUrlPath))}`;
        });
        this.styleUrlsCache.set(id, { styleUrls, matchedStyleUrls });
        return styleUrls;
    }
}
exports.StyleUrlsResolver = StyleUrlsResolver;
function getTextByProperty(name, properties) {
    return properties
        .filter((property) => property.getName() === name)
        .map((property) => property.getInitializer()?.getText().replace(/['"]/g, ''))
        .filter((url) => url !== undefined);
}
function getStyleUrls(code) {
    const project = new ts_morph_1.Project({ useInMemoryFileSystem: true });
    const sourceFile = project.createSourceFile('cmp.ts', code);
    const properties = sourceFile.getDescendantsOfKind(ts_morph_1.SyntaxKind.PropertyAssignment);
    const styleUrl = getTextByProperty('styleUrl', properties);
    const styleUrls = properties
        .filter((property) => property.getName() === 'styleUrls')
        .map((property) => property.getInitializer())
        .flatMap((array) => array.getElements().map((el) => el.getText().replace(/['"]/g, '')));
    return [...styleUrls, ...styleUrl];
}
exports.getStyleUrls = getStyleUrls;
function getTemplateUrls(code) {
    const project = new ts_morph_1.Project({ useInMemoryFileSystem: true });
    const sourceFile = project.createSourceFile('cmp.ts', code);
    const properties = sourceFile.getDescendantsOfKind(ts_morph_1.SyntaxKind.PropertyAssignment);
    return getTextByProperty('templateUrl', properties);
}
exports.getTemplateUrls = getTemplateUrls;
class TemplateUrlsResolver {
    templateUrlsCache = new Map();
    resolve(code, id) {
        const entry = this.templateUrlsCache.get(id);
        if (entry?.code === code) {
            return entry.templateUrlPaths;
        }
        const templateUrlPaths = getTemplateUrls(code).map((url) => `${url}|${(0, vite_1.normalizePath)((0, path_1.resolve)((0, path_1.dirname)(id), url).replace(/\\/g, '/'))}`);
        this.templateUrlsCache.set(id, { code, templateUrlPaths });
        return templateUrlPaths;
    }
}
exports.TemplateUrlsResolver = TemplateUrlsResolver;
//# sourceMappingURL=component-resolvers.js.map